// SQLColumns_Function.cpp  
// compile with: ODBC32.lib  
#include <stdio.h>
#include <stdlib.h>
#include <sql.h>
#include <sqlext.h>
#include <string.h>
#define ODBC_PUBLIC
#include "odbc.h"
#include <glib.h>
#include <time.h>

#include <stdarg.h>
#ifdef WIN32
#  include <windows.h>
#  define sleep(x) Sleep(x*1000)
#  include <crtdbg.h>
#else
#  include <unistd.h>
#  include <signal.h>
#endif


#define STR_LEN 128 + 1  
#define REM_LEN 254 + 1  

#define TRYODBC(h, ht, x)   {   RETCODE rc = x;\
                                if (rc != SQL_SUCCESS) \
                                { \
                                    HandleDiagnosticRecord (h, ht, rc); \
                                } \
                            }
                                
                                
void HandleDiagnosticRecord(SQLHANDLE hHandle, SQLSMALLINT hType,
        RETCODE RetCode) {
    SQLSMALLINT iRec = 0;
    SQLINTEGER iError;
    char szMessage[1000];
    char szState[SQL_SQLSTATE_SIZE + 1];

    if (RetCode == SQL_INVALID_HANDLE) {
        printf("Invalid handle!\n");
        return;
    }

    SQLRETURN ret;
    while ((ret = SQLGetDiagRec(hType, hHandle, ++iRec, szState, &iError,
            szMessage, (SQLSMALLINT) (sizeof(szMessage) / sizeof(char)),
            (SQLSMALLINT *) NULL)) == SQL_SUCCESS) {
        if (strncmp(szState, "01004", 5)) {
            printf("[%5.5s] %s (%d)\n", szState, szMessage, iError);
        }
    }
}
// Declare buffers for result set data  
SQLCHAR szSchema[STR_LEN];  
SQLCHAR szCatalog[STR_LEN];  
SQLCHAR szColumnName[STR_LEN];  
SQLCHAR szTableName[STR_LEN];  
SQLCHAR szTypeName[STR_LEN];  
SQLCHAR szRemarks[REM_LEN];  
SQLCHAR szColumnDefault[STR_LEN];  
SQLCHAR szIsNullable[STR_LEN];  

SQLUINTEGER ColumnSize;  
SQLUINTEGER BufferLength;  
SQLINTEGER CharOctetLength;  
SQLINTEGER OrdinalPosition;  

SQLSMALLINT DataType;  
SQLSMALLINT DecimalDigits;  
SQLSMALLINT NumPrecRadix;  
SQLSMALLINT Nullable;  
SQLSMALLINT SQLDataType;  
SQLSMALLINT DatetimeSubtypeCode;  

SQLHSTMT hstmt = NULL;  

// Declare buffers for bytes available to return  
SQLINTEGER cbCatalog;  
SQLINTEGER cbSchema;  
SQLINTEGER cbTableName;  
SQLLEN cbColumnName;  
SQLINTEGER cbDataType;  
SQLLEN cbTypeName;  
SQLLEN cbColumnSize;  
SQLLEN cbBufferLength;  
SQLINTEGER cbDecimalDigits;  
SQLINTEGER cbNumPrecRadix;  
SQLINTEGER cbNullable;  
SQLINTEGER cbRemarks;  
SQLINTEGER cbColumnDefault;  
SQLINTEGER cbSQLDataType;  
SQLINTEGER cbDatetimeSubtypeCode;  
SQLINTEGER cbCharOctetLength;  
SQLINTEGER cbOrdinalPosition;  
SQLINTEGER cbIsNullable;  

int main() {  
   SQLHENV henv;  
   SQLHDBC hdbc;  
   SQLHSTMT hstmt = 0;  
   SQLRETURN retcode;  

   //SQLCHAR conStr[] ="DSN=myodbc8w;DRIVER={/home/qsdb/code/install/mysql-connector-odbc/lib/libmyodbc8w.so};SERVER=127.0.0.1;port=60017;DATABASE=test;UID=test;PASSWORD=test;CharSet=utf8mb4;Protocol=TCPIP;";
   SQLCHAR conStr[] ="DSN=myodbc8w;DRIVER={/home/qsdb/code/install/mysql-connector-odbc/lib/libmyodbc8w.so};SERVER=127.0.0.1;port=60017;DATABASE=test;UID=test;PASSWORD=test;CharSet=utf8mb4;Protocol=TCPIP;no_i_s=1;";
   retcode = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);  
   retcode = SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER*)SQL_OV_ODBC3, 0);   
   retcode = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);   
   retcode = SQLSetConnectAttr(hdbc, SQL_LOGIN_TIMEOUT, (SQLPOINTER)5, 0);  
   retcode = SQLDriverConnect(hdbc, NULL, conStr, SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT);
   TRYODBC(hdbc, SQL_HANDLE_DBC, retcode);
   retcode = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);  
   //retcode = SQLColumns(hstmt, NULL, 0, (SQLCHAR*)"test", SQL_NTS, (SQLCHAR*)"test_gbk", SQL_NTS, NULL, 0);  

   TRYODBC(hstmt, SQL_HANDLE_STMT,
            retcode = SQLColumns(hstmt, (SQLCHAR*)"test", SQL_NTS, NULL, 0, (SQLCHAR*)"test_gbk", SQL_NTS, NULL, 0));

   printf("Get retcode:%d, SQLColumns Result:\n", retcode);
   if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {  
      // Bind columns in result set to buffers  
      SQLBindCol(hstmt, 4, SQL_C_CHAR, (SQLCHAR*)szColumnName, STR_LEN, &cbColumnName);  
      SQLBindCol(hstmt, 6, SQL_C_CHAR, (SQLCHAR*)szTypeName, STR_LEN, &cbTypeName);  
      SQLBindCol(hstmt, 7, SQL_C_SLONG, (SQLUINTEGER*)&ColumnSize, 0, &cbColumnSize);  
      SQLBindCol(hstmt, 8, SQL_C_SLONG, (SQLUINTEGER*)&BufferLength, 0, &cbBufferLength);  

      while (SQL_SUCCESS == retcode) {  
         retcode = SQLFetch(hstmt);  
         if (retcode == SQL_NO_DATA_FOUND) {
            break;
         }
         printf("column:%s, type:%s, buffer_len:%ld, cbBufferLength:%ld\n", szColumnName, szTypeName, BufferLength, cbBufferLength);
         /*  
         if (retcode == SQL_ERROR || retcode == SQL_SUCCESS_WITH_INFO)  
            0;   // show_error();  
         if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO)  
            0;   // Process fetched data  
         else  
            break;  
        */  
      }  
   }  
}
